require 'minitest_helper'

describe Kojak::Caller do
  let(:obj) do
    Dummy.new
  end

  let(:method) do
    :dummy1
  end

  let(:caller) do
    Kojak::Caller.new(obj.method(method))
  end

  describe "initialize" do
    it "should gather base information about the method" do
      caller[:name].must_equal "#{obj.class.name}##{method}"
      caller[:method].must_equal obj.method(method)
      caller[:location].must_equal obj.method(method).source_location
      caller[:pid].must_equal Process.pid
      caller[:thread_id].must_equal Thread.object_id
      caller[:thread].must_equal Thread.current
    end
  end

  describe "call" do
    describe "when normal call" do
      it "should store args" do
        args = {:foo => 1}
        caller.call(args)
        caller[:args].must_equal [args]
        caller[:block].must_be_nil
        caller[:block_given].must_equal false
      end

      it "should return same result as method would" do
        res = caller.call
        res.must_equal 1
        caller[:result].must_equal res
      end

      it "should measure execution time" do
        res = caller.call
        caller[:realtime].must_be ">", 0
      end
    end

    describe "when block given" do
      it "should store info about the block" do
        block = proc {}
        caller.call(&block)
        assert_equal caller[:block], block
        caller[:block_given].must_equal true
      end
    end

    describe "when error raised from method" do
      let(:method) do
        :dummy2
      end

      it "should capture it and re raise" do
        res = nil
        err = assert_raises ArgumentError do
          res = caller.call(1,2,3,4)
        end
        caller[:err].must_equal err
        caller[:result].must_be_nil
      end
    end
  end
end
