require 'minitest_helper'
require 'stringio'

describe Kojak::Printer::Basic do
  let(:out) do
    StringIO.new
  end

  let(:printer) do
    Kojak::Printer::Basic.new(out)
  end

  let(:text) do
    "foo"
  end

  describe "write" do
    it "should write given string to the output" do
      printer.write(text)
      out.seek(0)
      out.read.must_equal(text)
    end

    describe "interpolation" do
      it "should interpolate text" do
        printer.write("%s", text)
        out.seek(0)
        out.read.must_equal(text.inspect)
      end

      describe "with objects" do
        let(:obj) do
          {:foo => 1}
        end

        it "should pretty print interpolated objects" do
          printer.write("%s", obj)
          out.seek(0)
          out.read.must_equal(JSON.pretty_generate(obj))
        end
      end
    end
  end

  describe "puts" do
    it "should write given string and newline to the output" do
      printer.expects(:write).with(text + "\n")
      printer.puts(text)
    end
  end
end
